/*
Copyright (C) 2007-2008, Parrot Foundation.
$Id: main.c 39628 2009-06-17 14:39:27Z NotFound $

=head1 NAME

src/main.c - the Entry Point to Parrot Programs

=head1 DESCRIPTION

Start Parrot

=head2 Functions

=over 4

=cut

*/

#include <stdio.h>

#include "parrot/parrot.h"
#include "parrot/embed.h"
#include "parrot/imcc.h"

/*

=item C<int main(int argc, char * argv[])>

The entry point from the command line into Parrot.

=cut

*/

#include <setjmp.h>

static jmp_buf shell_parrot_exit_jmp;
static int shell_parrot_exit_status;
static Interp* Parent_Interp;

typedef PMC* (*parrot_dynops_load)(PARROT_INTERP);

PMC* Parrot_lib_math_ops_load(PARROT_INTERP);
PMC* Parrot_lib_math_ops_cg_load(PARROT_INTERP);
PMC* Parrot_lib_math_ops_cgp_load(PARROT_INTERP);
PMC* Parrot_lib_math_ops_switch_load(PARROT_INTERP);
PMC* Parrot_lib_obscure_ops_load(PARROT_INTERP);
PMC* Parrot_lib_obscure_ops_cg_load(PARROT_INTERP);
PMC* Parrot_lib_obscure_ops_cgp_load(PARROT_INTERP);
PMC* Parrot_lib_obscure_ops_switch_load(PARROT_INTERP);

parrot_dynops_load dynops[] =
{
    Parrot_lib_math_ops_load,
    Parrot_lib_math_ops_cg_load,
    Parrot_lib_math_ops_cgp_load,
    Parrot_lib_math_ops_switch_load,
    Parrot_lib_obscure_ops_load,
    Parrot_lib_obscure_ops_cg_load,
    Parrot_lib_obscure_ops_cgp_load,
    Parrot_lib_obscure_ops_switch_load
};

void
Parrot_rtems_exit (int status)
{
    shell_parrot_exit_status = status;
    longjmp (shell_parrot_exit_jmp, 1);
}

int
shell_parrot_main(int argc, char * argv[])
{
    const char *sourcefile;
    Interp     *interp;
    int         status;

    if (!Parent_Interp)
    {
        int dop;
        
        /* internationalization setup */
        /* setlocale(LC_ALL, ""); */
        PARROT_BINDTEXTDOMAIN(PACKAGE, LOCALEDIR);
        PARROT_TEXTDOMAIN(PACKAGE);

        Parrot_set_config_hash();

        Parent_Interp = Parrot_new(NULL);

        for (dop = 0;
             dop < (sizeof(dynops) / sizeof(parrot_dynops_load));
             dop++)
        {
            dynops[dop](Parent_Interp);
        }
    }
    
    interp = Parrot_new(Parent_Interp);
    imcc_initialize(interp);

    /* We parse the arguments, but first store away the name of the Parrot
       executable, since parsing destroys that and we want to make it
       available. */
    Parrot_set_executable_name(interp, Parrot_str_new(interp, argv[0], 0));

    sourcefile = parseflags(interp, &argc, &argv);
    if (setjmp (shell_parrot_exit_jmp) == 0)
        status = imcc_run(interp, sourcefile, argc, argv);
    else
        status = shell_parrot_exit_status;

    /* Clean-up after ourselves */
    Parrot_destroy(interp);
    return status;
}

/*

=back

=head1 SEE ALSO

F<compilers/imcc/main.c>, unfortunately.

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
